// This file is part of Waiting for Godot.
// Copyright 2025 by rjf.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "CommonData.h"
#include "resource.h"
#include "Shlwapi.h"

extern NPP::FuncItem menuDefinition[];  // Defined in Plugin.cpp
extern int menuItem_ToggleWatcher;      // Defined in Plugin.cpp


namespace {

HWND watcherPanel = 0;

DialogStretch stretch;

void updateWatcherPanelUnconditional() {
    SetDlgItemInt(watcherPanel, IDC_STATUS_INSERTS, data.insertsCounted, true);
    SetDlgItemInt(watcherPanel, IDC_STATUS_DELETES, data.deletesCounted, true);
}


INT_PTR CALLBACK watcherDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM) {

    switch (uMsg) {

    case WM_DESTROY:
        return TRUE;

    case WM_INITDIALOG:
        stretch.setup(hwndDlg);
        npp(NPPM_MODELESSDIALOG, MODELESSDIALOGADD, hwndDlg);   // a docking dialog must be a modeless dialog
        return TRUE;

    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDCANCEL:
            DestroyWindow(hwndDlg);
            watcherPanel = 0;
            return TRUE;
        case IDOK:
            SetFocus(plugin.currentScintilla());                // make Enter key return to active edit window
            return TRUE;
        }
        return FALSE;

    case WM_SHOWWINDOW:
        if (wParam) updateWatcherPanelUnconditional();
        npp(NPPM_SETMENUITEMCHECK, menuDefinition[menuItem_ToggleWatcher]._cmdID, wParam ? 1 : 0);
        return FALSE;

    case WM_SIZE:
        stretch.adjust(IDC_WATCHER_TEXT, 1).adjust(IDC_WATCHER_RESULT, 1);
        return FALSE;

    }

    return FALSE;
}

}


void updateWatcherPanel() { if (watcherPanel && IsWindowVisible(watcherPanel)) updateWatcherPanelUnconditional(); }

void toggleWatcherPanel() {
    if (watcherPanel)
        npp(IsWindowVisible(watcherPanel) ? NPPM_DMMHIDE : NPPM_DMMSHOW, 0, watcherPanel);
    else {
        watcherPanel = CreateDialog(plugin.dllInstance, MAKEINTRESOURCE(IDD_WATCHER), plugin.nppData._nppHandle, watcherDialogProc);
        NPP::tTbData dock;
        dock.hClient       = watcherPanel;
        dock.pszName       = L"Watcher (Waiting for Godot)";  // title bar text (caption in dialog is replaced)
        dock.dlgID         = menuItem_ToggleWatcher;          // zero-based position in menu to recall dialog at next startup
        dock.uMask         = DWS_DF_CONT_RIGHT;               // first time display will be docked at the right
        dock.pszModuleName = L"Waiting for Godot.dll";        // plugin module name
        npp(NPPM_DMMREGASDCKDLG, 0, &dock);
    }
}
